-- Windows interface for PragmAda Software Engineering's Mine Detector Game
-- using Claw (demo version, downloadable from www.rrsoftware.com)
-- This is a derived work under the Gnu General Public License, see below.
-- tmoran@acm.org 12/12/01
--
with Claw,
     Claw.Canvas,
     Claw.Basic_Window,
     Claw.Frame_Window,
     Claw.Menus.Bar;
-- Mine Detector Game
-- Copyright (C) 2001 by PragmAda Software Engineering.  All rights reserved.
-- **************************************************************************
--
-- Provide the user interface
--
with Field;
package User_If is
   procedure Display_Count (Count : in Field.Valid_Count; Stepped : in Boolean; Cell : in Field.Cell_Location);

   procedure Display_Mark (Cell : in Field.Cell_Location); -- Display a marked cell

   procedure Display_Mine (Cell : in Field.Cell_Location); -- Display a stepped-on mine

   procedure Display_Blank (Cell : in Field.Cell_Location); -- Display a blank cell

   procedure Display_To_Go (To_Go : in Integer); -- Display # of mines still to mark

   procedure Reset_Screen; -- Return to start of game condition

-- now add stuff to use Claw

   procedure Run;

private

  type Position_Lists is array(Integer range <>) of Claw.Int;

  subtype Cell_String is String (1 .. 1);

  type Button_States is record
    Text : Cell_String;
    Active : Boolean;
  end record;

  type Button_State_Arrays is Array(Field.Valid_Row, Field.Valid_Column) of Button_States;

  type Game_Board_Type is new Claw.Frame_Window.Frame_Window_Type with record
    Menu : Claw.Menus.Bar.Menu_Bar_Type;
    Game_Over_Area,
    Mines_Left_Area: Claw.Rectangle_Type;
    Cell_Top : Position_Lists(Field.Valid_Row'first .. Field.Valid_Row'last+1);
    Cell_Left : Position_Lists(Field.Valid_Column'first .. Field.Valid_Column'last+1);
    Ready : Boolean := False;  -- main program sets up while not Ready and
    -- When routines' task does nothing.  Then main program sets Ready := True
    -- after which it does nothing, but When routines modify the following:
    Mines_Left : Integer; -- really mines-marks, which can be negative
    Button_State : Button_State_Arrays;
  end record;

  procedure When_Draw (Window : in out Game_Board_Type;
                       Easel  : in out Claw.Canvas.Basic_Canvas_Type'Class;
                       Erase_Background : in Boolean;
                       Area   : in Claw.Rectangle_Type);
  -- called when part or all of game board needs to be (re)drawn

  procedure When_Command (Window : in out Game_Board_Type;
                          From   : in Claw.Command_Source_Type;
                          Command_Id : in Claw.Menu_Identifier_Type;
                          Unknown_Command : in out Boolean);
  -- called on user menu selection

  procedure When_Left_Button_Down(Window       : in out Game_Board_Type;
                            Mouse_Params : in     Claw.Mouse.Mouse_Params_Type;
                            Do_Default   : in out Boolean);
  -- called when left mouse button is clicked in game board Window

  procedure When_Right_Button_Down(Window       : in out Game_Board_Type;
                            Mouse_Params : in     Claw.Mouse.Mouse_Params_Type;
                            Do_Default   : in out Boolean);
  -- called when right mouse button is clicked in game board Window


  type Rule_Box_Type is new Claw.Basic_Window.Basic_Window_Type with null record;

  procedure When_Resize (Window        : in out Rule_Box_Type;
                         Resize_Params : in     Claw.Codes.Resize_Params_Type;
                         Do_Default    : in out Boolean);
  -- called when rule box Window is resized

end User_If;
--
-- This is free software; you can redistribute it and/or modify it under
-- terms of the GNU General Public License as published by the Free Software
-- Foundation; either version 2, or (at your option) any later version.
-- This software is distributed in the hope that it will be useful, but WITH
-- OUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-- or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-- for more details. Free Software Foundation, 59 Temple Place - Suite
-- 330, Boston, MA 02111-1307, USA.
